@@ -4,6 +4,7 @@
 import operator
 import types
 import datetime
+from collections import defaultdict
 
 
 def in_(a, b):
@@ -15,23 +16,17 @@ def in_(a, b):
     else:
         return operator.contains(b, a)
 
-OPERATORS = {
-    '=': operator.eq,
-    '>': operator.gt,
-    '<': operator.lt,
-    '<=': operator.le,
-    '>=': operator.ge,
-    '!=': operator.ne,
-    'in': in_,
-    'not in': lambda a, b: not in_(a, b),
-    # Those operators are not supported (yet ?)
-    'like': lambda a, b: True,
-    'ilike': lambda a, b: True,
-    'not like': lambda a, b: True,
-    'not ilike': lambda a, b: True,
-    'child_of': lambda a, b: True,
-    'not child_of': lambda a, b: True,
-}
+OPERATORS = defaultdict(lambda: lambda a, b: True)
+OPERATORS.update({
+        '=': operator.eq,
+        '>': operator.gt,
+        '<': operator.lt,
+        '<=': operator.le,
+        '>=': operator.ge,
+        '!=': operator.ne,
+        'in': in_,
+        'not in': lambda a, b: not in_(a, b),
+        })
 
 
 def locale_part(expression, field_name, locale_name='id'):
@@ -46,8 +41,7 @@ def locale_part(expression, field_name, locale_name='id'):
 def is_leaf(expression):
     return (isinstance(expression, (list, tuple))
         and len(expression) > 2
-        and isinstance(expression[1], basestring)
-        and expression[1] in OPERATORS)
+        and isinstance(expression[1], basestring))
 
 
 def constrained_leaf(part, boolop=operator.and_):
@@ -141,7 +135,7 @@ def eval_domain(domain, context, boolop=operator.and_):
             eval_domain(domain[1:], context, boolop))
 
 
-def localize_domain(domain, field_name=None):
+def localize_domain(domain, field_name=None, strip_target=False):
     "returns only locale part of domain. eg: langage.code -> code"
     if domain in ('AND', 'OR', True, False):
         return domain
@@ -154,10 +148,12 @@ def localize_domain(domain, field_name=None):
         locale_name = 'id'
         if isinstance(domain[2], basestring):
             locale_name = 'rec_name'
+        n = 3 if strip_target else 4
         return [locale_part(domain[0], field_name, locale_name)] \
-            + list(domain[1:3]) + list(domain[4:])
+            + list(domain[1:n]) + list(domain[4:])
     else:
-        return [localize_domain(part, field_name) for part in domain]
+        return [localize_domain(part, field_name, strip_target)
+            for part in domain]
 
 
 def simplify(domain):
@@ -608,10 +604,12 @@ def test_localize():
     assert localize_domain(domain, 'x') == [['y', 'child_of', [1]]]
 
     domain = [['x.id', '=', 1, 'y']]
-    assert localize_domain(domain, 'x') == [['id', '=', 1]]
+    assert localize_domain(domain, 'x', False) == [['id', '=', 1, 'y']]
+    assert localize_domain(domain, 'x', True) == [['id', '=', 1]]
 
     domain = [['a.b.c', '=', 1, 'y', 'z']]
-    assert localize_domain(domain, 'x') == [['b.c', '=', 1, 'z']]
+    assert localize_domain(domain, 'x', False) == [['b.c', '=', 1, 'y', 'z']]
+    assert localize_domain(domain, 'x', True) == [['b.c', '=', 1, 'z']]
 
 if __name__ == '__main__':
     test_simple_inversion()
